/**
 * @file 6LoWPAN_C_click.c
 * @brief 6LoWPAN C click ARM example
 *
 ******************************************************************************/

/*******************************************************************************
********************************************************************************
* Includes
********************************************************************************
*******************************************************************************/

#include <stdint.h>
#include "__C6LP_Driver.h"

/*******************************************************************************
********************************************************************************
* Preprocessor Constants
********************************************************************************
*******************************************************************************/

// Application mode: 1 - receiver; 0 - transmitter
// Change this value to switch between receiver and transmitter modes
// of the application.
#define RECEIVER 1

// Application parameters
#define RF_CHANNEL                25      // 2.4 GHz RF channel

// BasicRF address definitions
#define PAN_ID                0x2017
#define SWITCH_ADDR           0x1234
#define LIGHT_ADDR            0x5678
#define APP_PAYLOAD_LENGTH        1
#define LIGHT_TOGGLE_CMD          0

// Application states
#define IDLE                      0
#define SEND_CMD                  1

// Application role
#define NONE                      0
#define SWITCH                    1
#define LIGHT                     2
#define APP_MODES                 2

/*******************************************************************************
********************************************************************************
* Memory Declarations
********************************************************************************
*******************************************************************************/

// LEDs
sbit LED0 at DIO_P8OUT.B0;
sbit LED1 at DIO_P8OUT.B1;

// 6LoWPAN click pins
sbit C6LP_RESET at DIO_P5OUT.B6;
sbit C6LP_VREG_EN at DIO_P2OUT.B7;

sbit C6LP_SPI_CS at DIO_P5OUT.B0;
sbit C6LP_SPI_MISO at DIO_P1IN.B7;

sbit C6LP_GPIO0 at DIO_P6IN.B0;

#if (!RECEIVER)
static uint8_t flag;
#endif

static uint8_t pTxData[APP_PAYLOAD_LENGTH];
static uint8_t pRxData[APP_PAYLOAD_LENGTH];
static T_C6LP_cfg config;

/*******************************************************************************
********************************************************************************
* Public Functions
********************************************************************************
*******************************************************************************/

#if (!RECEIVER)
void port1InterruptHandler() iv IVT_INT_PORT1 ics ICS_AUTO
{
    if (P1IFG0_bit)
    {
        P1IFG0_bit = 0;

        flag = 1;
    }
}
#endif

void port2InterruptHandler() iv IVT_INT_PORT2 ics ICS_AUTO
{
    if (P2IFG4_bit)
    {
        P2IFG4_bit = 0;

        C6LP_rxFrmDoneIsr();
    }
}

void C6LP_userDisableRxInterrupt(void)
{
    // Clear the IRQ
    NVIC_IntDisable(IVT_INT_PORT2);
    P2IFG4_bit = 0;
}

void C6LP_userEnableRxInterrupt(void)
{
    NVIC_IntEnable(IVT_INT_PORT2);
}

uint8_t C6LP_spiRxTx(uint8_t x)
{
    return SPI5_Read(x);
}

void main(void)
{
    // Config basicRF
    config.panId = PAN_ID;
    config.channel = RF_CHANNEL;
    config.ackRequest = TRUE;

    GPIO_Digital_Output(&DIO_P1IN, _GPIO_PINMASK_6);
    SPI5_Init_Advanced(_SPI_MASTER, 8, _SPI_CLOCKSOURCE_SMCLK, _SPI_MSB_FIRST,
        _SPI_PHASE_DATA_CAPTURE_FIRST_CHANGE_NEXT, _SPI_CLK_IDLE_LOW,
        _SPI_3PIN, &_GPIO_MODULE_SPIB0_A576);
        
    GPIO_Digital_Output(&DIO_P5IN, _GPIO_PINMASK_6); // RF_RESET
    GPIO_Digital_Output(&DIO_P2IN, _GPIO_PINMASK_7); // RF_VREG_EN
    GPIO_Digital_Output(&DIO_P5IN, _GPIO_PINMASK_0); // RF_SPI_CS

    GPIO_Digital_Input(&DIO_P6IN, _GPIO_PINMASK_0); // RF_GPIO0
    
    // LEDs
    GPIO_Digital_Output(&DIO_P8IN, _GPIO_PINMASK_0 | _GPIO_PINMASK_1);
    
    EnableInterrupts(); // Enable interrupts

#if (!RECEIVER)
    // button
    GPIO_Digital_Input(&DIO_P1IN, _GPIO_PINMASK_0);
    
    // Enable button interrupt
    NVIC_IntEnable(IVT_INT_PORT1); // Enable PORT1 interrupt
    P1IES0_bit = 0;                // Rising Edge will trigger interrupt
    P1IE0_bit = 1;                 // Set mask to pin0
#endif

    // Enable rx interrupt
    NVIC_IntEnable(IVT_INT_PORT2);
    P2IFG4_bit = 0;
    P2IE4_bit = 1;  // Set mask to pin4
    P2IES4_bit = 0; // Rising Edge will trigger interrupt

    // LEDs to 0
    DIO_P8OUT.B0 = 0;
    DIO_P8OUT.B1 = 0;
    
    Delay_ms(100);

#if RECEIVER
    // Initialize BasicRF
    config.myAddr = LIGHT_ADDR;
    if (C6LP_init(&config) == FAILED)
        LED1 = 1;
        
    C6LP_receiveOn();
    EnableInterrupts();
    
    while (1)
    {
        while (!C6LP_isPacketReady());

        if (C6LP_receive(pRxData, APP_PAYLOAD_LENGTH, 0) > 0)
        {
            if (pRxData[0] == LIGHT_TOGGLE_CMD)
            {
                LED0 = !LED0;
            }
        }
    }
#else
    pTxData[0] = LIGHT_TOGGLE_CMD;

    // Initialize BasicRF
    config.myAddr = SWITCH_ADDR;
    if (C6LP_init(&config) == FAILED)
        LED1 = 1;
    
    // Keep Receiver off when not needed to save power
    C6LP_receiveOff();

    flag = 0;
    EnableInterrupts();
    while (1)
    {
        if (flag)
        {
            flag = 0;
            LED0 = !LED0;
            C6LP_sendPacket(LIGHT_ADDR, pTxData, APP_PAYLOAD_LENGTH);
            EnableInterrupts();
        }
    }
#endif
}